/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */
 
 package com.eviware.soapui;

import java.io.File;
import java.io.FileInputStream;

import javax.swing.JOptionPane;
import javax.swing.UIManager;

import com.eviware.soapui.config.SoapuiSettingsDocumentConfig;
import com.eviware.soapui.impl.wsdl.actions.iface.tools.support.SwingToolHost;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.inspectors.attachments.AttachmentsInspectorFactory;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.inspectors.httpheaders.HttpHeadersInspectorFactory;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.inspectors.registry.XmlInspectorRegistry;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.inspectors.script.ScriptInspectorFactory;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.inspectors.ssl.SSLInspectorFactory;
import com.eviware.soapui.model.settings.Settings;
import com.eviware.soapui.model.workspace.Workspace;
import com.eviware.soapui.support.UISupport;
import com.eviware.x.form.XFormFactory;
import com.eviware.x.impl.swing.SwingFileDialogs;
import com.eviware.x.impl.swing.SwingFormFactory;

public class SwingSoapUICore extends DefaultSoapUICore
{
	public SwingSoapUICore()
	{
		super();
	}
	
	public SwingSoapUICore( String settingsFile )
	{
		super( null, settingsFile );
	}

	public void prepareUI()
	{
		UISupport.setToolHost( new SwingToolHost() );
      XFormFactory.Factory.instance = new SwingFormFactory();
	}
	
	public void afterStartup(Workspace workspace)
	{
		XmlInspectorRegistry inspectorRegistry = XmlInspectorRegistry.getInstance();
		inspectorRegistry.addFactory( new ScriptInspectorFactory() );
		inspectorRegistry.addFactory( new HttpHeadersInspectorFactory() );
		inspectorRegistry.addFactory( new AttachmentsInspectorFactory() );
		inspectorRegistry.addFactory( new SSLInspectorFactory() );
		
		addExternalActions( getRoot() == null ? "actions" : getRoot() + File.separatorChar + "actions",	
					getExtensionClassLoader() );
	}

	@Override
	protected Settings initSettings( String fileName )
	{
		if( !new File( fileName ).exists() )
		{
			try
			{
				fileName = importSettingsOnStartup( fileName );
			}
			catch( Exception e )
			{
				e.printStackTrace();
			}
		}
		
		return super.initSettings( fileName );
	}
	
	protected String importSettingsOnStartup( String fileName ) throws Exception
	{
		javax.swing.UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		
		if( JOptionPane.showConfirmDialog( null, "Missing soapUI Settings, import from existing installation?", 
					"Import Preferences", JOptionPane.YES_NO_OPTION ) == JOptionPane.YES_OPTION )
		{
			while( true )
			{
				File settingsFile = SwingFileDialogs.openFile( null, "Import Preferences", ".xml", "soapUI settings XML", fileName );
				if( settingsFile != null )
				{
					try
					{
						SoapuiSettingsDocumentConfig.Factory.parse( settingsFile );
						log.info( "imported soapui-settings from [" + settingsFile.getAbsolutePath() + "]" );
						return settingsFile.getAbsolutePath();
					}
					catch( Exception e )
					{
						if( JOptionPane.showConfirmDialog( null, "Error loading settings from [" + settingsFile.getAbsolutePath() + "]\r\nspecify another?", 
									"Error Importing", JOptionPane.OK_CANCEL_OPTION, JOptionPane.ERROR_MESSAGE ) == JOptionPane.CANCEL_OPTION )
						{
							break;
						}
					}
				}
			}
		}
		
		return fileName;
	}

	private void addExternalActions( String folder, ClassLoader classLoader )
	{
		File[] actionFiles = new File( folder ).listFiles();
      if( actionFiles != null )
      {
	      for( File actionFile : actionFiles )
	      {
	      	if( actionFile.isDirectory() )
	      	{
	      		addExternalActions( actionFile.getAbsolutePath(), classLoader );
	      		continue;
	      	}
	      	
	      	if( !actionFile.getName().toLowerCase().endsWith( "-actions.xml" ))
	      		continue;
	      	
	      	try
				{
	      		log.info( "Adding actions from [" + actionFile.getAbsolutePath() + "]" );
					
					SoapUI.getActionRegistry().addConfig( new FileInputStream( actionFile ), 
								classLoader);
				}
				catch( Exception e )
				{
					SoapUI.logError( e );
				}
	      }
      }
	}
}
