/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.actions.testcase;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.xml.namespace.QName;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.WorkspaceImpl;
import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.impl.wsdl.WsdlProject;
import com.eviware.soapui.impl.wsdl.WsdlTestSuite;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.impl.wsdl.testcase.WsdlTestCase;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestStep;
import com.eviware.soapui.model.iface.Interface;
import com.eviware.soapui.model.project.Project;
import com.eviware.soapui.model.support.ModelSupport;
import com.eviware.soapui.support.SoapUIException;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.support.AbstractSoapUIAction;
import com.eviware.x.form.XFormDialog;
import com.eviware.x.form.XFormField;
import com.eviware.x.form.XFormFieldListener;
import com.eviware.x.form.support.ADialogBuilder;
import com.eviware.x.form.support.AField;
import com.eviware.x.form.support.AForm;
import com.eviware.x.form.support.AField.AFieldType;

/**
 * Clones a WsdlTestSuite
 * 
 * @author Ole.Matzura
 */

public class CloneTestCaseAction extends AbstractSoapUIAction<WsdlTestCase>
{
	private static final String CREATE_NEW_OPTION = "<Create New>";
	private XFormDialog dialog;

	public CloneTestCaseAction() 
   {
      super( "Clone TestCase", "Clones this TestCase" );
   }
	
   public void perform( WsdlTestCase testCase, Object param )
	{
   	if( dialog == null )
   	{
		   dialog = ADialogBuilder.buildDialog( Form.class );
		   dialog.getFormField( Form.PROJECT ).addFormFieldListener( new XFormFieldListener() {

				public void valueChanged( XFormField sourceField, String newValue, String oldValue )
				{
					if( newValue.equals( CREATE_NEW_OPTION ))
						dialog.setOptions( Form.TESTSUITE, new String[] {CREATE_NEW_OPTION} );
					else
					{
						Project project = SoapUI.getWorkspace().getProjectByName( newValue );
						dialog.setOptions( Form.TESTSUITE, 
								ModelSupport.getNames( project.getTestSuites(), new String[] {CREATE_NEW_OPTION} ));
					}
				}} );
   	}
   	
   	dialog.setValue( Form.NAME, "Copy of " + testCase.getName() );
   	WorkspaceImpl workspace = testCase.getTestSuite().getProject().getWorkspace();
		dialog.setOptions( Form.PROJECT, 
					ModelSupport.getNames( workspace.getOpenProjectList(), new String[] {CREATE_NEW_OPTION} ) );
		
		dialog.setValue( Form.PROJECT, testCase.getTestSuite().getProject().getName() );
		
		dialog.setOptions( Form.TESTSUITE, 
					ModelSupport.getNames( testCase.getTestSuite().getProject().getTestSuites(), new String[] {CREATE_NEW_OPTION} ) );
		
		dialog.setValue( Form.TESTSUITE, testCase.getTestSuite().getName() );
		boolean hasLoadTests = testCase.getLoadTestCount() > 0;
		dialog.setBooleanValue( Form.CLONE_LOADTESTS, hasLoadTests );
		dialog.getFormField( Form.CLONE_LOADTESTS ).setEnabled( hasLoadTests );
		
		if( dialog.show() )
		{
			String targetProjectName = dialog.getValue( Form.PROJECT );
			String targetTestSuiteName = dialog.getValue( Form.TESTSUITE );
			String name = dialog.getValue( Form.NAME );
			
			WsdlProject project = testCase.getTestSuite().getProject();
			WsdlTestSuite targetTestSuite = null;
			Set<WsdlInterface> requiredInterfaces = new HashSet<WsdlInterface>();
			
			// to another project project?
			if( !targetProjectName.equals( project.getName() ))
			{
				// get required interfaces
				for( int y = 0; y < testCase.getTestStepCount(); y++ )
				{
					WsdlTestStep testStep = testCase.getTestStepAt( y );
					requiredInterfaces.addAll( testStep.getRequiredInterfaces() );
				}
				
				project = ( WsdlProject ) workspace.getProjectByName( targetProjectName );
				if( project == null )
				{
					targetProjectName = UISupport.prompt( "Enter name for new Project", "Clone TestCase", "" );
					if( targetProjectName == null )
						return;
					
					try
					{
						project = workspace.createProject( targetProjectName );
					}
					catch( SoapUIException e )
					{
						UISupport.showErrorMessage( e );
					}
					
					if( project == null )
						return;
				}
				
				if( requiredInterfaces.size() > 0 && project.getInterfaceCount() > 0 )
				{
					Map<QName,WsdlInterface> bindings = new HashMap<QName,WsdlInterface>();
					for( WsdlInterface iface : requiredInterfaces )
					{
						bindings.put( iface.getBindingName(), iface );
					}
					
					for( Interface iface : project.getInterfaces() )
					{
						bindings.remove( iface.getBindingName() );
					}

					requiredInterfaces.retainAll( bindings.values() );
				}
				
				if( requiredInterfaces.size() > 0 )
				{
					String msg = "Target project [" + targetProjectName  +"] is missing required interfaces;\r\n\r\n";
					for( WsdlInterface iface : requiredInterfaces )
					{
						msg += iface.getName() + " [" + iface.getBindingName() + "]\r\n";
					}
					msg += "\r\nThese will be cloned to the targetProject as well";
					
					if( !UISupport.confirm( msg, "Clone TestCase" ))
						return;
					
					for( WsdlInterface iface : requiredInterfaces )
					{
						project.importInterface( iface, true );
					}
				}
			}
			 
			targetTestSuite = project.getTestSuiteByName( targetTestSuiteName );
	      if( targetTestSuite == null )
	      {
	      	targetTestSuiteName = UISupport.prompt( "Specify name for new TestSuite", "Clone TestCase", 
	      				"Copy of " + testCase.getTestSuite().getName() );
	      	if( targetTestSuiteName == null )
	      		return;
	      	
	      	targetTestSuite = project.addNewTestSuite( targetTestSuiteName );
	      }
			
			WsdlTestCase newTestCase = targetTestSuite.importTestCase( testCase, name, -1, dialog.getBooleanValue( Form.CLONE_LOADTESTS ) );
			UISupport.select( newTestCase );
			
			if( dialog.getBooleanValue( Form.MOVE ))
			{
			   testCase.getTestSuite().removeTestCase( testCase );
			}
		}
   }
   
   @AForm(description = "Specify target Project/TestSuite and name of cloned TestCase", name = "Clone TestCase",
   			helpUrl=HelpUrls.CLONETESTCASE_HELP_URL, icon=UISupport.TOOL_ICON_PATH)
	protected interface Form
	{
		@AField( name="TestCase Name", description = "The name of the cloned TestCase", type=AFieldType.STRING )
		public final static String NAME = "TestCase Name";

		@AField( name="Target TestSuite", description = "The target TestSuite for the cloned TestCase", type=AFieldType.ENUMERATION )
		public final static String TESTSUITE = "Target TestSuite";

		@AField( name="Target Project", description = "The target Project for the cloned TestCase", type=AFieldType.ENUMERATION )
		public final static String PROJECT = "Target Project";
		
		@AField( name="Clone LoadTests", description = "Clone contained LoadTests", type=AFieldType.BOOLEAN )
		public final static String CLONE_LOADTESTS = "Clone LoadTests";
		
		@AField( name="Move instead", description = "Moves the selected TestCase instead of copying", type=AFieldType.BOOLEAN )
		public final static String MOVE = "Move instead";
	}
}
