/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.request.components;

import com.eviware.soapui.impl.wsdl.panels.request.components.editor.XmlDocument;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.XmlEditorView;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.XmlInspector;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.inspectors.registry.RequestInspectorFactory;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.inspectors.registry.XmlInspectorFactory;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.inspectors.registry.XmlInspectorRegistry;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.views.registry.RequestEditorViewFactory;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.views.registry.XmlEditorViewFactory;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.views.registry.XmlEditorViewRegistry;
import com.eviware.soapui.model.ModelItem;

/**
 * XmlEditor for the request of a WsdlRequest
 * 
 * @author ole.matzura
 */

public class RequestMessageXmlEditor<T extends ModelItem> extends SoapMessageXmlEditor<T>
{
	public RequestMessageXmlEditor( XmlDocument xmlDocument, T modelItem  )
	{
		super( xmlDocument, modelItem );
		
		XmlEditorViewFactory[] editorFactories = XmlEditorViewRegistry.getInstance().getFactoriesOfType( 
					RequestEditorViewFactory.class );
		
		for( XmlEditorViewFactory factory : editorFactories )
		{
			RequestEditorViewFactory f = ( RequestEditorViewFactory ) factory;
			XmlEditorView editorView = f.createRequestEditorView( this, modelItem );
			if( editorView != null )
				addEditorView( editorView);
		}
		
		XmlInspectorFactory[] inspectorFactories = XmlInspectorRegistry.getInstance().getFactoriesOfType( 
					RequestInspectorFactory.class );
		
		for( XmlInspectorFactory factory : inspectorFactories )
		{
			RequestInspectorFactory f = ( RequestInspectorFactory ) factory;
			XmlInspector inspector = f.createRequestInspector( this, modelItem );
			if( inspector != null )
				addInspector( inspector);
		}
	}
}
