/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.request.components.editor.inspectors.attachments;

import java.beans.PropertyChangeListener;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

import com.eviware.soapui.impl.wsdl.AttachmentContainer;
import com.eviware.soapui.impl.wsdl.WsdlAttachmentPart;
import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.impl.wsdl.mock.WsdlMockResponse;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.XmlEditor;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.XmlInspector;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.inspectors.registry.RequestInspectorFactory;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.inspectors.registry.ResponseInspectorFactory;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.model.iface.Attachment;
import com.eviware.soapui.model.iface.MessagePart;

public class AttachmentsInspectorFactory implements RequestInspectorFactory, ResponseInspectorFactory
{
	public static final String INSPECTOR_ID = "Attachments";

	public String getInspectorId()
	{
		return INSPECTOR_ID;
	}

	public XmlInspector createRequestInspector( XmlEditor editor, ModelItem modelItem )
	{
		if( modelItem instanceof WsdlRequest )
			return new AttachmentsInspector( ( AttachmentContainer ) modelItem );
		else if( modelItem instanceof WsdlMockResponse )
			return new AttachmentsInspector( new MockRequestAttachmentsContainer( (WsdlMockResponse)modelItem ) );
		
		return null;
	}

	public XmlInspector createResponseInspector( XmlEditor editor, ModelItem modelItem )
	{
		if( modelItem instanceof WsdlRequest )
			return new AttachmentsInspector( new ResponseAttachmentsContainer( (WsdlRequest)modelItem ) );
		else if( modelItem instanceof WsdlMockResponse )
			return new AttachmentsInspector( ( WsdlMockResponse ) modelItem );
		
		return null;
	}
	
	private static class ResponseAttachmentsContainer implements AttachmentContainer
	{
		private final WsdlRequest request;

		public ResponseAttachmentsContainer( WsdlRequest request )
		{
			this.request = request;
		}

		public void addAttachmentsChangeListener( PropertyChangeListener listener )
		{
			request.addPropertyChangeListener( WsdlRequest.RESPONSE_PROPERTY, listener );
		}

		public Attachment attachFile( File file, boolean cache )
		{
			return null;
		}

		public Attachment getAttachmentAt( int index )
		{
			return request.getResponse() == null ? null : request.getResponse().getAttachments()[index];
		}

		public int getAttachmentCount()
		{
			return request.getResponse() == null ? 0 : request.getResponse().getAttachments().length;
		}

		public WsdlAttachmentPart getAttachmentPart( String partName )
		{
			return null;
		}

		public Attachment[] getAttachments()
		{
			return request.getResponse() == null ? null : request.getResponse().getAttachments();
		}

		public Attachment[] getAttachmentsForPart( String partName )
		{
			return request.getResponse() == null ? null : request.getResponse().getAttachmentsForPart( partName );
		}

		public WsdlAttachmentPart[] getDefinedAttachmentParts()
		{
			MessagePart[] responseParts = request.getResponseParts();
			
			List<WsdlAttachmentPart> result = new ArrayList<WsdlAttachmentPart>();
			
			for( MessagePart part : responseParts )
				if( part instanceof WsdlAttachmentPart )
					result.add( ( WsdlAttachmentPart ) part );
			
			return result.toArray( new WsdlAttachmentPart[result.size()] );
		}

		public boolean isMtomEnabled()
		{
			return request.isMtomEnabled();
		}

		public boolean isMultipartEnabled()
		{
			return request.isMultipartEnabled();
		}

		public boolean isReadOnly()
		{
			return true;
		}

		public void removeAttachment( Attachment attachment )
		{
		}

		public void removeAttachmentsChangeListener( PropertyChangeListener listener )
		{
			request.removePropertyChangeListener( WsdlRequest.RESPONSE_PROPERTY, listener );
		}
	}

	private static class MockRequestAttachmentsContainer implements AttachmentContainer
	{
		private final WsdlMockResponse request;

		public MockRequestAttachmentsContainer( WsdlMockResponse request )
		{
			this.request = request;
		}

		public void addAttachmentsChangeListener( PropertyChangeListener listener )
		{
			request.addPropertyChangeListener( WsdlMockResponse.MOCKRESULT_PROPERTY, listener );
		}

		public Attachment attachFile( File file, boolean cache )
		{
			return null;
		}

		public Attachment getAttachmentAt( int index )
		{
			return request.getMockResult() == null ? null : request.getMockResult().getMockRequest().getRequestAttachments()[index];
		}

		public int getAttachmentCount()
		{
			return request.getMockResult() == null ? 0 : request.getMockResult().getMockRequest().getRequestAttachments().length;
		}

		public WsdlAttachmentPart getAttachmentPart( String partName )
		{
			return null;
		}

		public Attachment[] getAttachments()
		{
			return request.getMockResult() == null ? null : request.getMockResult().getMockRequest().getRequestAttachments();
		}

		public Attachment[] getAttachmentsForPart( String partName )
		{
			return null;
		}

		public WsdlAttachmentPart[] getDefinedAttachmentParts()
		{
			return null;
		}

		public boolean isMtomEnabled()
		{
			return request.isMtomEnabled();
		}

		public boolean isMultipartEnabled()
		{
			return request.isMultipartEnabled();
		}

		public boolean isReadOnly()
		{
			return true;
		}

		public void removeAttachment( Attachment attachment )
		{
		}

		public void removeAttachmentsChangeListener( PropertyChangeListener listener )
		{
			request.removePropertyChangeListener(  WsdlMockResponse.MOCKRESULT_PROPERTY, listener );
		}
	}
		
}
