/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.testcase;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import javax.swing.text.Document;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.actions.support.ShowOnlineHelpAction;
import com.eviware.soapui.impl.wsdl.actions.testcase.TestCaseOptionsAction;
import com.eviware.soapui.impl.wsdl.panels.support.ProgressBarTestCaseAdapter;
import com.eviware.soapui.impl.wsdl.panels.testcase.actions.SetCredentialsAction;
import com.eviware.soapui.impl.wsdl.panels.testcase.actions.SetEndpointAction;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.impl.wsdl.testcase.WsdlTestCase;
import com.eviware.soapui.impl.wsdl.testcase.WsdlTestCaseRunner;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.model.support.PropertiesMap;
import com.eviware.soapui.model.support.TestRunListenerAdapter;
import com.eviware.soapui.model.testsuite.LoadTestRunner;
import com.eviware.soapui.model.testsuite.TestRunContext;
import com.eviware.soapui.model.testsuite.TestRunner;
import com.eviware.soapui.model.testsuite.TestStep;
import com.eviware.soapui.model.testsuite.TestStepResult;
import com.eviware.soapui.monitor.support.TestMonitorListenerAdapter;
import com.eviware.soapui.support.DocumentListenerAdapter;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.swing.SwingActionDelegate;
import com.eviware.soapui.support.components.JComponentInspector;
import com.eviware.soapui.support.components.JInspectorPanel;
import com.eviware.soapui.support.components.JUndoableTextArea;
import com.eviware.soapui.support.swing.ComponentBag;
import com.eviware.soapui.ui.support.ModelItemDesktopPanel;

/**
 * WsdlTestCase desktop panel
 * 
 * @author Ole.Matzura
 */

public class WsdlTestCaseDesktopPanel extends ModelItemDesktopPanel<WsdlTestCase>
{
	private JProgressBar progressBar;
	private TestStepList testStepList;
	private InternalTestRunListener testRunListener = new InternalTestRunListener();
	private JButton runButton;
	private JButton cancelButton;
	private TestRunner runner;
	private JButton setEndpointButton;
	private JButton setCredentialsButton;
	private JButton optionsButton;
	private ComponentBag stateDependantComponents = new ComponentBag();
	private TestCaseLog testCaseLog;
	private JToggleButton loopButton;
	@SuppressWarnings( "unused" )
	private ProgressBarTestCaseAdapter progressBarAdapter;
	private InternalTestMonitorListener testMonitorListener;
	public boolean canceled;
	private JTextArea descriptionArea;

	public WsdlTestCaseDesktopPanel( WsdlTestCase testCase )
	{
		super( testCase );

		buildUI();

		setPreferredSize( new Dimension( 250, 500 ) );
		setRunningState();

		testCase.addTestRunListener( testRunListener );
		progressBarAdapter = new ProgressBarTestCaseAdapter( progressBar, testCase );
		testMonitorListener = new InternalTestMonitorListener();

		SoapUI.getTestMonitor().addTestMonitorListener( testMonitorListener );
	}

	/**
	 * There are three states: - enabled, no testcases or testschedules running -
	 * enabled, standalone testcase running - disabled, testschedule is running
	 */

	private void setRunningState()
	{
		stateDependantComponents.setEnabled( !SoapUI.getTestMonitor().hasRunningLoadTest( getModelItem() ) );
	}

	private void buildUI()
	{
		JPanel panel = new JPanel( new BorderLayout() );

		panel.add( buildToolbar(), BorderLayout.PAGE_START );
		panel.add( buildRunnerBar(), BorderLayout.CENTER );

		add( panel, BorderLayout.NORTH );

		JInspectorPanel inspectorPanel = new JInspectorPanel( buildTestStepList() );
		inspectorPanel.addInspector( new JComponentInspector( buildTestLog(), "TestCase Log", "TestCase Execution Log", true ) );
		inspectorPanel.setDefaultDividerLocation( 0.7F );
		inspectorPanel.setCurrentInspector( "TestCase Log" );
		
//		JSplitPane splitPane = UISupport.createVerticalSplit();
//		splitPane.setTopComponent( buildTestStepList() );
//		splitPane.setBottomComponent( buildTestLog() );
//		splitPane.setDividerLocation( 0.7 );
//		splitPane.setDividerLocation( 250 );

		add( inspectorPanel, BorderLayout.CENTER );
		add( new JLabel( "--" ), BorderLayout.PAGE_END );
	}

	protected TestStepList getTestStepList()
	{
		return testStepList;
	}

	private JComponent buildTestLog()
	{
		testCaseLog = new TestCaseLog();
		stateDependantComponents.add( testCaseLog );
		return testCaseLog;
	}

	private JComponent buildTestStepList()
	{
		testStepList = new TestStepList( getModelItem() );
		stateDependantComponents.add( testStepList );

		JTabbedPane tabs = new JTabbedPane( JTabbedPane.TOP );
		tabs.addTab( "Test Steps", new JScrollPane( testStepList ) );
		tabs.addTab( "Description", buildDescriptionPanel() );

		// tabs.addTab( "Test Properties", new JPanel() );

		return UISupport.createTabPanel( tabs, false );
	}

	private Component buildDescriptionPanel()
	{
		JPanel panel = new JPanel( new BorderLayout() );
		descriptionArea = new JUndoableTextArea( getModelItem().getDescription() );
		descriptionArea.getDocument().addDocumentListener( new DocumentListenerAdapter()
		{
			public void update( Document document )
			{
				getModelItem().setDescription( descriptionArea.getText() );
			}
		} );

		panel.setBorder( BorderFactory.createEmptyBorder( 2, 2, 2, 2 ) );
		panel.add( new JScrollPane( descriptionArea ), BorderLayout.CENTER );

		return panel;
	}

	private Component buildToolbar()
	{
		JToolBar toolbar = UISupport.createToolbar();

		runButton = UISupport.createToolbarButton( new RunTestCaseAction() );
		optionsButton = UISupport.createToolbarButton( SwingActionDelegate.createDelegate(
					TestCaseOptionsAction.SOAPUI_ACTION_ID, getModelItem(), null, "/options.gif" ) );
		optionsButton.setText( null );
		cancelButton = UISupport.createToolbarButton( new CancelRunTestCaseAction(), false );

		loopButton = new JToggleButton( UISupport.createImageIcon( "/loop.gif" ) );
		loopButton.setPreferredSize( UISupport.getPreferredButtonSize() );
		loopButton.setToolTipText( "Loop TestCase continuously" );

		setCredentialsButton = UISupport.createToolbarButton( new SetCredentialsAction( getModelItem() ) );
		setEndpointButton = UISupport.createToolbarButton( new SetEndpointAction( getModelItem() ) );

		stateDependantComponents.add( runButton );
		stateDependantComponents.add( optionsButton );
		stateDependantComponents.add( cancelButton );
		stateDependantComponents.add( setCredentialsButton );
		stateDependantComponents.add( setEndpointButton );

		addToolbarActions( toolbar );
		
		toolbar.add( Box.createHorizontalGlue() );
		toolbar.add( UISupport.createToolbarButton( new ShowOnlineHelpAction( HelpUrls.TESTCASEEDITOR_HELP_URL ) ) );

		return toolbar;
	}

	protected void addToolbarActions( JToolBar toolbar )
	{
		toolbar.add( runButton );
		toolbar.add( cancelButton );
		toolbar.add( loopButton );
		toolbar.add( setCredentialsButton );
		toolbar.add( setEndpointButton );
		toolbar.add( optionsButton );
	}

	private Component buildRunnerBar()
	{
		progressBar = new JProgressBar( 0, getModelItem().getTestStepCount() );
		return UISupport.createProgressBarPanel( progressBar, 10, false );
	}

	private final class InternalTestMonitorListener extends TestMonitorListenerAdapter
	{
		public void loadTestStarted( LoadTestRunner runner )
		{
			setRunningState();
		}

		public void loadTestFinished( LoadTestRunner runner )
		{
			setRunningState();
		}
	}

	public class InternalTestRunListener extends TestRunListenerAdapter
	{
		private SimpleDateFormat dateFormat;

		public InternalTestRunListener()
		{
			dateFormat = new SimpleDateFormat( "yyyy-MM-dd HH:mm:ss.SSS" );
		}

		public void beforeRun( TestRunner testRunner, TestRunContext runContext )
		{
			if( SoapUI.getTestMonitor().hasRunningLoadTest( getModelItem() ) )
				return;

			runButton.setEnabled( false );
			cancelButton.setEnabled( true );
			testStepList.setEnabled( false );
			testStepList.setSelectedIndex( -1 );
			testCaseLog.clear();

			testCaseLog.addText( "Test started at " + dateFormat.format( new Date() ) );
		}

		public void beforeStep( TestRunner testRunner, TestRunContext runContext )
		{
			if( SoapUI.getTestMonitor().hasRunningLoadTest( getModelItem() ) )
				return;

			TestStep testStep = runContext.getCurrentStep();
			testStepList.setSelectedValue( testStep, true );
		}

		public void afterRun( TestRunner testRunner, TestRunContext runContext )
		{
			if( SoapUI.getTestMonitor().hasRunningLoadTest( getModelItem() ) )
				return;

			WsdlTestCaseRunner wsdlRunner = ( WsdlTestCaseRunner ) testRunner;

			if( testRunner.getStatus() == TestRunner.Status.CANCELED )
				testCaseLog.addText( "Test canceled [" + testRunner.getReason() + "], time taken = "
							+ wsdlRunner.getTimeTaken() );
			else if( testRunner.getStatus() == TestRunner.Status.FAILED )
				testCaseLog.addText( "Test failed [" + wsdlRunner.getError() + "], time taken = "
							+ wsdlRunner.getTimeTaken() );
			else 
				testCaseLog.addText( "Test finished with status [" + testRunner.getStatus() + "], time taken = " + wsdlRunner.getTimeTaken() );

			runner = null;

			JToggleButton loopButton = ( JToggleButton ) runContext.getProperty( "loopButton" );
			if( loopButton != null && loopButton.isSelected() && testRunner.getStatus() == TestRunner.Status.FINISHED )
			{
				SwingUtilities.invokeLater( new Runnable()
				{
					public void run()
					{
						runTestCase();
					}
				} );
			}
			else
			{
				WsdlTestCaseDesktopPanel.this.afterRun();
			}
		}

		public void afterStep( TestRunner testRunner, TestRunContext runContext, TestStepResult stepResult )
		{
			if( SoapUI.getTestMonitor().hasRunningLoadTest( getModelItem() ) )
				return;

			testCaseLog.addTestStepResult( stepResult );
		}
	}

	protected void runTestCase()
	{
		if( canceled )
		{
			// make sure state is correct
			runButton.setEnabled( true );
			cancelButton.setEnabled( false );
			testStepList.setEnabled( true );
			return;
		}

		PropertiesMap properties = new PropertiesMap();
		properties.put( "loopButton", loopButton );
		runner = getModelItem().run( properties, true );
	}

	public class RunTestCaseAction extends AbstractAction
	{
		public RunTestCaseAction()
		{
			putValue( Action.SMALL_ICON, UISupport.createImageIcon( "/run_testcase.gif" ) );
			putValue( Action.SHORT_DESCRIPTION, "Runs this testcase" );
		}

		public void actionPerformed( ActionEvent e )
		{
			canceled = false;
			runTestCase();
		}
	}

	public class CancelRunTestCaseAction extends AbstractAction
	{
		public CancelRunTestCaseAction()
		{
			putValue( Action.SMALL_ICON, UISupport.createImageIcon( "/stop_testcase.gif" ) );
			putValue( Action.SHORT_DESCRIPTION, "Stops running this testcase" );
		}

		public void actionPerformed( ActionEvent e )
		{
			if( runner != null )
				runner.cancel( "canceled in UI" );

			canceled = true;
		}
	}

	public boolean onClose( boolean canCancel )
	{
		if( canCancel )
		{
			if( runner != null && runner.getStatus() == TestRunner.Status.RUNNING )
			{
				Boolean retval = UISupport.confirmOrCancel( "Cancel running TestCase?", "Cancel Run" );

				if( retval == null )
					return false;
				if( retval.booleanValue() )
				{
					runner.cancel( null );
				}
			}
		}
		else
		{
			if( runner != null && runner.getStatus() == TestRunner.Status.RUNNING )
			{
				runner.cancel( null );
			}
		}

		SoapUI.getTestMonitor().removeTestMonitorListener( testMonitorListener );
		getModelItem().removeTestRunListener( testRunListener );
		testStepList.release();
		progressBarAdapter.release();
		
		return release();
	}

	public boolean dependsOn( ModelItem modelItem )
	{
		return modelItem == getModelItem() || modelItem == getModelItem().getTestSuite()
					|| modelItem == getModelItem().getTestSuite().getProject();
	}

	protected void afterRun()
	{
		runButton.setEnabled( true );
		cancelButton.setEnabled( false );
		testStepList.setEnabled( true );
	}
}
