/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.support;

import java.util.StringTokenizer;

public class StringUtils
{
	public static String unquote( String str )
	{
		int length = str == null ? -1 : str.length();
		if( str == null || length == 0 )
			return str;

		if( length > 1 && str.charAt( 0 ) == '\"' && str.charAt( length - 1 ) == '\"' )
		{
			str = str.substring( 1, length - 1 );
		}

		return str;
	}

	public static boolean isNullOrEmpty( String str )
	{
		return str == null || str.length() == 0 || str.trim().length() == 0;
	}

	public static int parseInt( String str, int defaultValue )
	{
		if( isNullOrEmpty( str ) )
			return defaultValue;

		try
		{
			return Integer.parseInt( str );
		}
		catch( NumberFormatException e )
		{
			return defaultValue;
		}
	}

	public static String normalizeSpace( String str )
	{
		if( !isNullOrEmpty( str ) )
		{
			StringTokenizer st = new StringTokenizer( str );
			if( st.hasMoreTokens() )
			{

				StringBuffer sb = new StringBuffer( str.length() );
				while( true )
				{
					sb.append( st.nextToken() );
					if( st.hasMoreTokens() )
					{
						sb.append( ' ' );
					}
					else
					{
						break;
					}
				}
				return sb.toString();
			}
			else
			{
				return "";
			}
		}
		else
		{
			return str;
		}
	}

	public static boolean hasContent( String str )
	{
		return str != null && str.trim().length() > 0;
	}
}
