/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.submit.filters;

import junit.framework.TestCase;

import com.eviware.soapui.impl.wsdl.WsdlSubmitContext;

public class PropertyExpansionTestCase extends TestCase
{
   public void testExpansion() throws Exception
   {
   	WsdlSubmitContext context = new WsdlSubmitContext( null );
   	
   	context.setProperty( "test", "value" );
   	
   	assertEquals( "value", PropertyExpansionRequestFilter.expandProperties( context, "${test}" ));
   	assertEquals( "value", PropertyExpansionRequestFilter.expandProperties( context, "${#test}" ));
   	assertEquals( " value ", PropertyExpansionRequestFilter.expandProperties( context, " ${test} " ));
   	assertEquals( "${testa}", PropertyExpansionRequestFilter.expandProperties( context, "${testa}" ));
   	assertEquals( "valuevalue", PropertyExpansionRequestFilter.expandProperties( context, "${test}${test}" ));

   	context.setProperty( "testa", "" );
   	assertEquals( "", PropertyExpansionRequestFilter.expandProperties( context, "${testa}" ));
   }
   
   public void testRecursiveExpansion() throws Exception
   {
   	WsdlSubmitContext context = new WsdlSubmitContext( null );
   	
   	context.setProperty( "test", "value" );
   	context.setProperty( "testexp", "${test}" );
   	
   	assertEquals( "value", PropertyExpansionRequestFilter.expandProperties( context, "${testexp}" ));

   	context.setProperty( "exp", "${exp}" );
   	assertEquals( "${exp}", PropertyExpansionRequestFilter.expandProperties( context, "${exp}" ));
   }
   
   public void testNestedExpansion() throws Exception
   {
   	WsdlSubmitContext context = new WsdlSubmitContext( null );
   	
   	context.setProperty( "test", "value" );
   	context.setProperty( "testexp", "${test}" );
   	context.setProperty( "exp", "exp" );
   	
   	assertEquals( "value", PropertyExpansionRequestFilter.expandProperties( context, "${test${exp}}" ));
   	
   	context.setProperty( "id", "123" );
   	context.setProperty( "testxml", "<test><value id=\"123\">hello</value></test>" );
   	assertEquals( "hello", 
   				PropertyExpansionRequestFilter.expandProperties( context, "${#testxml#//value[@id=${id}]/text()}" ));

   	context.setProperty( "testxpath", "//value[@id=${id}]/text()" );
   	assertEquals( "hello", 
   				PropertyExpansionRequestFilter.expandProperties( context, "${#testxml#${testxpath}}" ));
   }
   
   public void testXPathExpansion() throws Exception 
   {
   	WsdlSubmitContext context = new WsdlSubmitContext( null );
   	
   	context.setProperty( "test", "<test><value>hello</value></test>" );
   	assertEquals( "hello", PropertyExpansionRequestFilter.expandProperties( context, "${#test#//value/text()}" ));
   }
}
